;
;  benchmark program  : arc_1024.asm
; 
;  benchmark suite    : DSP-kernel
;
;  description        : benchmarking of an integer FFT
;                       Radix 2, In-place, Decimation-in-time Complex FFT Macro
;
;                       This macro uses a special pipelining technique which 
;			overlaps butterflies to obtain faster overall execution,
;			at a small cost in extra program length. 
;
;			This macro performs a complete Fast Fourier Transform 
;			(FFT) on complex data.  The basic algorithm is the 
;			Decimation-in-time (DIT), Radix 2 FFT algorithm using 
;			24 bit fixed-point arithmetic.  The algorithm uses
;			a sine-cosine lookup table for the FFT coefficients
;			(twiddle factors).
;
;			FFTR2CC can be called to perform any FFT from 16-32768 
;			points.  Simply call it with the arguments of number of 
;			FFT points, location of the data array and location of 
;			the sine-cosine table.  All register initialization is 
;			performed by this macro.  
;
;			However, the macro assumes that registers which should
;			not be altered by the FFT have already been saved by 
;			the main program.  This allows the user to fit the FFT 
;			macro into his application and thus control the context 
;			switching overhead.
;
;			No data scaling is performed and no overflow detection 
;			is done. Modifications to this routine could allow it to
;			be used with the scaling modes and thus allow dynamic 
;			scaling for each FFT pass.
;
;			All data and coefficients are complex, with the real 
;			part in X Data memory and the imaginary part in Y Data 
;			memory.  For an N point FFT, the data buffer requires 
;			N X Data and N Y Data memory locations.
;
;			The algorithm is performed "in-place", meaning that only
;			one data buffer is required for both input and output 
;			data.  The input data is assumed to be in normal
;			(time-sequential) order and the output is in 
;			bit-reversed order.  By using the reverse-carry
;			address modifier and a separate output data buffer, 
;			the output data may be easily unscrambled. Other methods
;			also exist to unscramble the output data without a 
;			separate output data buffer.
;
;			The FFTR2CC macro uses "twiddle factors" (-cosine and 
;			-sine tables) stored in data memory.  For maximum speed,
;			the FFT macro performs a lookup table operation to get 
;			new sine and cosine values for each group of 
;			butterflies. A SINCOS macro is available to generate 
;			these tables.  For an N point FFT, N/2 X Data and N/2
;		 	Y Data locations are required. 
;
;			The speed increase is obtained by performing the first
;			two passes in one combined operation without storing the
;			results of the first pass.  The first two passes do not
;			require any multiplies since the coefficients are -1, 0
;			or +1.  The last pass is also split out from the triple
;			nested DO loop and given a separate DO loop.  The reason
;		 	this is faster is that the inner loop always starts with
;			a loop count of 1 on the  last pass.  Note that the 
;			separate last pass DO loop uses different addressing 
;			modes to increment through the butterflies, thus 
;			avoiding outer loops.  Additional details are included 
;			in the source file.
;
;  func. verification : 
;
;  organization       : Aachen University of Technology - IS2
;                     : DSP Tools Group
;                     : phone   : +49(241)807887
;                     : fax     : +49(241)8888195
;                     : e-mail  : zivojnov@ert.rwth-aachen.de
;
;  author             : Juan Martinez Velarde
;
;  history            : 22-02-94 - assembler reference code creation
;                       23-02-94 - assembler reference code profiling
;             
	section	arc_1024
	opt so,nomd,cex,mu,contc,lb,msw,cc,nopp,ck,msw,mex
	page 132,66,3,3

;*** DSP56000/1 Motorola 1.11 GNU 1.37.1

        include 'sinegen'
        include 'sincos'
	include 'fftr2cc'

start   equ     $100
points  equ     1024		;number of FFT points
data    equ     $400
coef    equ     $800
mag1    equ     @pow(2.0,-8.0)
mag2    equ     @pow(2.0,-8.0)


	org	y:
LC5
	dc	't','i','m','e',$20,$3D,$20,$25,'d',$0A,$00


	org	p:
	global	Fmain
Fmain
	move	ssh,y:(r6)+
	move	r0,y:(r6)+
	
	nop
	move r6,r7	

;
; Generate two tone test input.
;
        org     x:data
        sinegen points,mag1,0.16666,0.0
        org     y:data
        sinegen points,mag2,0.235,0.0
;
; Generate FFT coefficients (twiddle factors).
;
        sincos  points,coef
;

;	
; Program starts here
;
	org	p:


; **************************************************
ASM_APP_0
	START_PROFILING:
ASM_APP_1

	move 	y:F__time,a
	move 	a1,y:(r7)
; **************************************************

	fftr2cc points,data,coef

; **************************************************
	move	y:(r7),b
	move	y:F__time,a
	sub b,a

ASM_APP_2
	END_PROFILING:
ASM_APP_3
; **************************************************

	move	#-1,m0
	move	m0,m1
	move	m0,m2
	move	m0,m3
	move	m0,m4
	move	m0,m5
	move	m0,m6
	move 	r7,r6
	nop
	move	#LC5,r1
	move	a1,y:(r6)
	move	y:(r6),a
	move	a1,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move	(r6)-
	move	(r6)-
	move	(r6)-
	move	(r6)-
	move	y:(r6),r0
 	clr 	a	(r6)-
	nop
	move	y:(r6),ssh
	rts

	endsec
