; **************************************************
; /*
;  * benchmark program:   fir_arc.asm
;  * 
;  * benchmark suite:     DSP-kernel
;  * 
;  * description:         fir - filter benchmarking
;  *
;  *	This FIR filter reads the input sample
;  *	from the memory location Y:input
;  *	and writes the filtered output sample
;  *	to the memory location Y:output
;
;  *	The samples are stored in the X memory
;  *	The coefficients are stored in the Y memory
;  *
;  *         X MEMORY                               Y MEMORY
;  *
;  *        |        |                             |        |
;  *   R0   |--------|                             |--------|
;  * +----->|  X(n)  |                      +----->|  c(0)  |
;  * |  t   |--------|                      |t,t+T |--------|
;  * |      | X(n-1) |                      |      |  c(1)  |
;  * |      |--------|                      |      |--------|
;  * |      |        |                      |      |        |
;  * |      |        |                      |      |        |
;  * |      |        |                      |      |        |
;  * |      |        |                      |      |        |
;  * |      |--------|                      |      |--------|
;  * +----->|X(n-k+1)|  X(n+1)              +<-----| c(k-1) |
;  *  t+T   |--------|                             |--------|
;  *        |        |                             |        |
;  *
;  *
;  *                            C(0)                      
;  *                            ___          ___
;  *  x(n)                     /   \        /   \         y(n)
;  *  -------------+----------|  X  |----->|  +  |--------->
;  *               |           \___/        \___/
;  *               |                          ^             k-1
;  *               |                          |             ____
;  *            +-----+                       |             \   '
;  *            |  T  |         C(1)          |      y(n)=  /___,c(p)x(n-p)
;  *            +-----+         ___           |             p=0
;  *               |           /   \          |
;  *               +----------|  X  |-------->+  
;  *               |           \___/          |
;  *            +-----+                       |
;  *            |  T  |         C(2)          |
;  *            +-----+         ___           |
;  *               |           /   \          |
;  *               +----------|  X  |-------->+   
;  *               |           \___/          |
;  *               |                          |
;  *               |                          |
;  *               |                          |
;  *               |                          |
;  *               |                          |
;  *               |                          |
;  *            +-----+                       |
;  *            |  T  |         C(K-1)        |
;  *            +-----+         ___           |
;  *               |           /   \          |
;  *               +----------|  X  |-------->+     
;  *                           \___/
;  *
;  *
;  *                          F I R
;  * 
;  * reference code:      Dr.Bub/56000/bench/1-56.asm
;  * 
;  * f. verification:     with printf function
;  * 
;  *  organization:        Aachen University of Technology - IS2 
;  *                       DSP Tools Group
;  *                       phone:  +49(241)807887 
;  *                       fax:    +49(241)8888195
;  *                       e-mail: zivojnov@ert.rwth-aachen.de 
;  *
;  * author:              Juan Martinez Velarde
;  * 
;  * history:             11-5-94 creation and profiling (Martinez Velarde)
;  * 
;  */

	section fir_arc
	opt so,nomd,cc


n	equ	16 	; number of taps of fir filter
input	equ	$ffe0 	; input value address
output	equ	$ffe1	; output value address

	org	x:$0	; samples in x memory
F___x0: dc 1
	dc 2
	dc 3
	dc 4
	dc 5
	dc 6
	dc 7
	dc 8
	dc 9
	dc 10
	dc 11
	dc 12
	dc 13
	dc 14
	dc 15
	dc 16

	org	y:$0	; coefficients in y memory
F___c0:	dc 1
	dc 1
	dc 1
	dc 1	
	dc 1
	dc 1
	dc 1
	dc 1	
	dc 1
	dc 1
	dc 1
	dc 1	
	dc 1
	dc 1
	dc 1
       	dc 1

LC0	dc	'output',$3D,$25,'i',$0A,$0A,$00

LC1	dc	'input',$3D,$25,'i',$0A,$0A,$00

LC2	dc	'x',$3D,$25,'i',$0A,$0A,$00

	org	p:$40
	global	Fmain
Fmain
	move	ssh,y:(r6)+	; save ssh
	move	m0,y:(r6)+	; save m0
	move	m4,y:(r6)	; save m4

	move 	#F___x0,r0
	move	#F___c0,r4	

	clr	a
	movep	#>100,y:input

; **************************************************
; print input value
;
	move	y:input,a1
	move	#LC1,r1
	move	a1,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-
; **************************************************

	move	#n-1,m0		; set modulo arithmetic
	move	m0,m4		; for both circular buffers
	nop

ASM_APP_0
	START_PROFILING:
ASM_APP_1

	clr	a 	x:(r0)+,x0	y:(r4)+,y0	
	
	rep	#n
	mac	x0,y0,a  x:(r0)+,x0  y:(r4)+,y0 ; y += *px++ * *pc++;	
L1
	asr a	(r0)-
	move	a0,a
	move	a,y:output              ;store filtered output
	movep	y:input,x:(r0)          ;store newest input
	
ASM_APP_2
	END_PROFILING:
ASM_APP_3

	move 	y:(r6)-,m4	; restore m4
	move	y:(r6),m0	; restore m0

; **************************************************
; print output value
;
	move	y:output,a
	move	#LC0,r1
	move	a1,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-
; **************************************************
; print state circular buffer
;
	move	#LC2,r1
	move	#F___x0,r0
	do	#n,L15
	move	x:(r0)+,a1
	move	a1,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-
L15
; **************************************************


	move	y:-(r6),ssh
	rts
	endsec

