 ;
 ;  benchmark program  : biq_N_arc.asm
 ;
 ;  benchmark suite    : DSP-kernel
 ;
 ;  description        : benchmarking of an iir biquad (N section)
 ;                       
 ;	The equations of the filter are:
 ;       w(n) =    x(n) - ai1*w(n-1) - ai2*w(n-2)
 ;       y(n) = b0*w(n) + bi1*w(n-1) + bi2*w(n-2)
 ;
 ; This routine implements a second order filter with a minimum of
 ; delay elements.  The difference equation relating the output
 ; to the input is:
 ;
 ;     y(n) = a1 * y(n-1) + a2 * y(n-2) + b0 * x(n) + b1 * x(n-1) + b2 * x(n-2)
 ;
 ;  with z transform:
 ;
 ;                         -1       -2    
 ;       Y(z)     b0 + b(1)z  + b(2)z
 ;     ------- = ----------------------
 ;       X(z)              -1       -2
 ;                1 - a(1)z  - a(2)z
 ;
 ; where:
 ;   x(n)  = biquad input sample at time nT
 ;   y(n)  = output of the biquad filter at time nT
 ;   a(n)  = filter coefficient n (magnitude less than one)
 ;   b(n)  = filter coefficient n (magnitude less than one)
 ;     T   = sample period
 ;
 ;		              w(n)
 ;   x (n)------(-)---------->-|->---b0----(+)-------> y(n)
 ;               A             |            A
 ;               |           |1/z|          |
 ;               |             | w(n-1)     |
 ;               |             v            |
 ;               |-<--a1-----<-|->---b1--->-|
 ;               |             |            |
 ;               |           |1/z|          |
 ;               |             | w(n-2)     |
 ;               |             v            |
 ;               |-<--a2-----<--->---b2--->-|
 ; 
 ;     The filter states w(n-1) and w(n-2) of each biquad are stored in X data locations
 ; 
 ;                                              
 ; For the above filter, the memory map  for  the  filter
 ;  states and the coefficients is shown below.
 ;
 ;          r0   
 ;          |
 ;          v
 ;      ---------------------------------------------
 ; X:   |        |        |        |        |       |
 ;      | w(n-2) | w(n-1) | w(n-2) | w(n-1) | ...   |   Filter States
 ;      ---------------------------------------------
 ;
 ;      --------------------------------------------------
 ; Y:   |  a(2)  |  a(1)  |  b(0)  |  b(1)  |  b(2) |
 ;      |  ----  | ------ | -----  |  ----- | ----- | ....   Filter Coefficients
 ;      |   2    |    2   |   2    |    2   |   2   |
 ;      --------------------------------------------------
 ;          ^
 ;          |
 ;          r3        
 ;
 ;              Memory Map for the Biquad Filter
 ;
 ; The address register r0 is initialized pointing to  the  first
 ; filter  state  and  r3  is  initialized pointing to the filter
 ; coefficients.  
 ;
 ;  reference code     : see dsp56100 family manual
 ;                           first set of benchmarks, p.B-10
 ;                       see implementing IIR/FIR Filters
 ;                           with Motorola's DSP56000/56001, 1991
 ;
 ;  func. verification : from separate computation
 ;
 ;  organization       : Aachen University of Technology - IS2
 ;                     : DSP Tools Group
 ;                     : phone   : +49(241)807887
 ;                     : fax     : +49(241)8888195
 ;                     : e-mail  : zivojnov@ert.rwth-aachen.de
 ;
 ;  author             : Juan Martinez Velarde
 ;
 ;  history            : 28-3-1994 assembler reference code creation and profiling
 ;

	section biq_N_arc
	opt so,nomd,cc
 

N	equ	4 		; number of biquads

        org     x:
datin   dc	$1000	    	; location in X memory of input sample
temp	ds	1		; reserve one location for a temporary variable
datout	ds	1	    	; location in X memory of output file 

states  ds      1        	; w(n-2) 1st biquad 
	ds	1		; w(n-1) 1st biquad 
        ds      1         	; w(n-2) 2nd biquad 
	ds	1		; w(n-1) 2nd biquad 
        ds      1         	; w(n-2) 3rd biquad 
	ds	1		; w(n-1) 3rd biquad 
        ds      1       	; w(n-2) 4th biquad 
	ds	1		; w(n-1) 4th biquad 

coeff   dc      $1		; a1/2 1st biquad
	dc	$1		; a2/2 1st biquad
	dc	$1		; b0/2 1st biquad
	dc	$1     		; b1/2 1st biquad
        dc	$1  		; b2/2 1st biquad
        dc      $1		; a1/2 2nd biquad
	dc	$1		; a2/2 2nd biquad
	dc	$1		; b0/2 2nd biquad
	dc	$1     		; b1/2 2nd biquad
        dc	$1  		; b2/2 2nd biquad
        dc      $1		; a1/2 3rd biquad
	dc	$1		; a2/2 3rd biquad
	dc	$1		; b0/2 3rd biquad
	dc	$1     		; b1/2 3rd biquad
        dc	$1  		; b2/2 3rd biquad
        dc      $1		; a1/2 4th biquad
	dc	$1		; a2/2 4th biquad
	dc	$1		; b0/2 4th biquad
	dc	$1     		; b1/2 4th biquad
        dc	$1  		; b2/2 4th biquad

	org	p:	
	global	Fmain
Fmain
	lea	(r2)+,r2
	move	ssh,x:(r2)+

        ori     #$08,mr
	move	#datin,r1	; get input sample
	move	#temp,r3
	move	x:(r1),a
	move	a,x:(r3)
	
	move	#temp,r1
        move    #states,r0	; point to filter states
        move    #coeff,r3	; point to filter coefficients (a2)
        move    #-1,n0

ASM_APP_0
	START_PROFILING:
ASM_APP_1
                
        do      #N,END_DOE			      ;for N biquads

	move	x:(r1),a			      ;a=biquad input sample
	move              x:(r0)+,y0    x:(r3)+,x0    ;y0=w(n-2), x0=a2/2,  
        mac     y0,x0,a   x:(r0)+n0,y1  x:(r3)+,x1    ;a =w(n-2)*(a2/2),y1=w(n-1),x1=a1/2
        mac     y1,x1,a   y1,x:(r0)+	              ;a+=w(n-1)*(a1/2),store w(n-2)=w(n-1)
	move	a,x:(r0)+n0    a,x1	     	      ;store w(n-1)= w(n) x1= w(n)
	move	x:(r3)+,y1		      	      ;x0=b0/2
	mac	y1,x1,a   x:(r0)+,y1    x:(r3)+,x1    ;a += b0/2*w(n), y1=old w(n-1), x1=b1/2
        mac     y1,x1,a   x:(r3)+,x1    	      ;a+=w(n-1)*b1/2, x1=b2/2
        macr    y0,x1,a   x:(r0)+,x0 		      ;a+=w(n-2)*b2/2, point to next state
	move	a,x:(r1)			      ;store biquad output temporary

END_DOE

	move	a,x:datout			      ; store output value

ASM_APP_2
	END_PROFILING:
ASM_APP_3

     	move	(r2)-
	move	x:(r2)-,ssh
	rts

	endsec


