 ;
 ;  benchmark program  : biquad_one_section.c
 ;
 ;  benchmark suite    : DSP-kernel
 ;
 ;  description        : benchmarking of an iir biquad (one section)
 ;                       
 ;	The equations of the filter are:
 ;       w(n) =    x(n) - ai1*w(n-1) - ai2*w(n-2)
 ;       y(n) = b0*w(n) + bi1*w(n-1) + bi2*w(n-2)
 ;
 ; This routine implements a second order filter with a minimum of
 ; delay elements.  The difference equation relating the output
 ; to the input is:
 ;
 ;     y(n) = a1 * y(n-1) + a2 * y(n-2) + b0 * x(n) + b1 * x(n-1) + b2 * x(n-2)
 ;
 ;  with z transform:
 ;
 ;                         -1       -2    
 ;       Y(z)     1 + b(1)z  + b(2)z
 ;     ------- = ----------------------
 ;       X(z)              -1       -2
 ;                1 - a(1)z  - a(2)z
 ;
 ; where:
 ;   x(n)  = input sample at time nT
 ;   y(n)  = output of the filter at time nT
 ;   a(n)  = filter coefficient n (magnitude less than one)
 ;   b(n)  = filter coefficient n (magnitude less than one)
 ;     T   = sample period
 ;
 ;		              w(n)
 ;   x (n)------(-)---------->-|->---b0----(+)-------> y(n)
 ;               A             |            A
 ;               |           |1/z|          |
 ;               |             | w(n-1)     |
 ;               |             v            |
 ;               |-<--a1-----<-|->---b1--->-|
 ;               |             |            |
 ;               |           |1/z|          |
 ;               |             | w(n-2)     |
 ;               |             v            |
 ;               |-<--a2-----<--->---b2--->-|
 ; 
 ;     The filter states w(n-1) and w(n-2) are stored in X data locations
 ; 
 ;                                              
 ; For the above filter, the memory map  for  the  filter
 ;  states and the coefficients is shown below.
 ;
 ;          r0   
 ;          |
 ;          v
 ;      -------------------
 ; X:   |        |        |
 ;      | w(n-1) | w(n-2) | Filter States
 ;      -------------------
 ;
 ;      ---------------------------------------------
 ; Y:   |  a(1)  |  a(2)  |  b(0)  |  b(2)  |  b(0) |
 ;      |  ----  | ------ | -----  |  ----- | ----- |   Filter Coefficients
 ;      |   2    |    2   |   2    |    2   |   2
 ;      ---------------------------------------------
 ;          ^
 ;          |
 ;          r3        
 ;
 ;              Memory Map for the Biquad Filter
 ;
 ; The address register r0 is initialized pointing to  the  first
 ; filter  state  and  r3  is  initialized pointing to the filter
 ; coefficients.  
 ;
 ;  reference code     : see dsp56100 family manual
 ;                           first set of benchmarks, p.B-10
 ;
 ;  func. verification : from separate computation
 ;
 ;  organization       : Aachen University of Technology - IS2
 ;                     : DSP Tools Group
 ;                     : phone   : +49(241)807887
 ;                     : fax     : +49(241)8888195
 ;                     : e-mail  : zivojnov@ert.rwth-aachen.de
 ;
 ;  author             : Juan Martinez Velarde
 ;
 ;  history            : 23-3-1994 assembler reference code creation and profiling
 ;

	section biq_one_arc
	opt so,nomd,cc

        org     x:
datin   dc	$1000	    	; location in X memory of input sample
datout	ds	1	    	; location in X memory of output file 
states  ds      1         	; w(n-2) 
	ds	1		; w(n-1)
coeff   dc      $1		; a1/2
	dc	$1		; a2/2
	dc	$1		; b0/2
	dc	$1     		; b1/2
        dc	$1  		; b2/2
	
	org	p:	
	global	Fmain
Fmain
	lea	(r2)+,r2
	move	ssh,x:(r2)+

        move    #states,r0     ; point to filter states
	move	#datin,r1      ; point to data input location
        move    #coeff,r3      ; point to filter coefficients 
	move	#-1,n0         ; linear arithmetic
	ori	#$08,MR
	nop
	

ASM_APP_0
	START_PROFILING:
ASM_APP_1

	clr	a              			x:(r3)+,y1	; clear accu, y1 = a1
	move	x:(r0)+,x0					; x0 = w(n-2)
	mac	x0,y1,a		x:(r0)+n0,y0	x:(r3)+,x1	; a += a1 * w(n-2), y0 = w(n-1), x1 = a2
	mac	x1,y0,a		y0,x:(r0)+	              	; a += a2 * w(n-1), w(n-2) = w(n-1)
		
	move	x:(r1),x1					; x1 = input sample
	add	x1,a		a,x:(r0)+n0			; a +=  x1, store w(n-1) = w(n)

	move	a,x1		x:(r3)+,y1			; x1 = w(n), y1 = b0
	mac	y1,x1,a	 	x:(r3)+,y1           		; a += b0 * w(n), y1 = b1
		
	move	x:(r0),x1					; x1 = old w(n-1)
	mac	y1,x1,a	 	x:(r3)+,y1			; a += b1 * w(n-1), y1= b2	
	mac	x0,y1,a						; a += b2 * w(n-2)

        move  	a,x:datout 		                	; output sample	

ASM_APP_2
	END_PROFILING:
ASM_APP_3

     	move	(r2)-
	move	x:(r2)-,ssh
	rts

	endsec



