; **************************************************
;
;  benchmark program:   lms_arc.asm
;   
;  benchmark suite:     DSP-kernel
;  
;  description:         integer lms - adaptive fir filter benchmarking
;
; Notation and symbols:
;
;   x(n)    - Input sample at time n.
;   d(n)    - Desired signal at time n.
;   y(n)    - FIR filter output at time n.
;   H(n)    - Filter coefficient vector at time n.
;                  H={c0,c1,c2,...,ck,...,c(N-1)}
;   X(n)    - Filter state variable vector at time N.
;                  X={x(n),x(n-1),....,x(n-N+1)}
;   Mu      - Adaptation gain.
;   N       - Number of coefficient taps in the filter. 
;
;   True LMS Algorithm     
;   Get input sample       
;   Save input sample      
;   Do FIR                 
;   Get d(n), find e(n)    
;   Update coefficients    
;   Output y(n)            
;   Shift vector X         

;   System equations:
;   e(n)=d(n)-H(n)X(n)     (FIR filter and error)
;   H(n+1)=H(n)+uX(n)e(n)  (Coefficient update)
;
; References:
;
;"Adaptive Digital Filters and Signal Analysis", Maurice G. Bellanger,
; Marcel Deker Inc. New York and Basel
;
;"The DLMS Algorithm Suitable for the Pipelined Realization of Adaptive
; Filters", Proc. IEEE ASSP Workshop, Academia Sinica, Beijing, 1986
;
; Note:
; The sections of code shown describe how to initialize all registers,
; filter an input sample and do the coefficient update. The instruc-
; tions relating to the filtering and coefficient update are shown as
; part of the benchmark. Instructions executed only once (for initial-
; ization) are not included in the benchmark.
;
;
;
;            Implementation of the true LMS on the DSP56100
; 
; 
; Memory map:
; 
;    Initial X                             H
;  x(n-0) x(n-1) x(n-2) ... x(n-N+1)       h0   h1   h2   ... hN-1
;   |                                      |
;   r0                                     r3
;                                          r1
; 
; 
;  reference code:      dsp56100 family manual
;                      first set of benchmarks, p.B-6.
; 
;  f. verification:     with printf function
; 
;  organization:        Aachen University of Technology - IS2 
;                       DSP Tools Group
;                       phone:  +49(241)807887 
;                       fax:    +49(241)8888195
;                       e-mail: zivojnov@ert.rwth-aachen.de 
; 
;  author:              Juan Martinez Velarde
; 
;  history:             30-3-94 assembler reference code creation and profiling (Martinez Velarde)

	section lms_arc
	opt	so,nomd,cc

N	equ 16

	org	x: 	; samples  and coefficients in x memory

F___x0: dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1
	dc 1

F___h1:	dc 1
	dc 1
	dc 1
	dc 1	
	dc 1
	dc 1
	dc 1
	dc 1	
	dc 1
	dc 1
	dc 1
	dc 1	
	dc 1
	dc 1
	dc 1
F___hn:	dc 1

input:  dc 6
desired: dc 3
output: dc 1
delta:  dc 1

	org	p:

;Implementation of the true LMS on the DSP56100

	global	Fmain
Fmain
	move	ssh,x:(r2)+

	move	m0,x:(r2)+
	move	m1,x:(r2)+
	move	n0,x:(r2)+

        move    #F___x0,r0                   ;start of X
        move    #F___h1,r3                   ;coefficients
        move    #N-1,m0                      ;mod N
        move    #-2,n0                       ;adjustment for filtering
        move    m0,m1                        ;mod N

START_PROFILING:

        move    x:input,y0          ;get input sample
	move	x:desired,y1	    ;get desired output in y1
	move	x:delta,x0	    ;get delta

        clr     b         y0,x:(r0)+                ;save x(n)
        move              x:(r3)+,x1                ;get c0
        rep     #N-1                                ;do fir
        mac     y0,x1,b   x:(r0)+,y0  x:(r3)+,x1    ;mac, update pointers 
        macr    y0,x1,b       	                    ;last tap and round

	neg	b			; negate fir output
	add	y1,b			; error e(n) = d(n) - y(n) 
	move	b,y1			; load result in y1

        move    #F___h1,r3                 ;coefficients
        move    r3,r1                      ;coefficients
        mpy     y1,x1,a       x:(r0)+,y0   ;perform multiplication e*delta
					   ;get x(n)
        move    a,x0          x:(r3)+,a    ;load result in x0, a = c0

        do      #N,_coefupdate                   ;do the coefficient update
        macr    x0,y0,a   x:(r0)+,y0  x:(r3)+,x1 ;macr, update pointers
        tfr     x1,a      a,x:(r1)+                 ;copy new coeff, take next 
_coefupdate
	
	move	a,x:output			    ;store output
        move              x:(r0)+n0,y0              ;update r0
        move              x:(r3)-,y0                ;update r3 

END_PROFILING:

	move	x:-(r2),n0
	move	x:-(r2),m1
	move	x:-(r2),m0

	move	x:-(r2),ssh
	rts
	endsec